@@ -1,4 +1,4 @@
-/** Copyright 2011 Thorsten Wißmann. All rights reserved.
+/** Copyright 2011-2012 Thorsten Wißmann. All rights reserved.
  *
  * This software is licensed under the "Simplified BSD License".
  * See LICENSE for details */
@@ -8,6 +8,7 @@
 #include "globals.h"
 #include "utils.h"
 #include "hook.h"
+#include "ewmh.h"
 #include "ipc-protocol.h"
 #include "settings.h"
 #include "layout.h"
@@ -51,6 +52,7 @@ char* g_layout_names[] = {
     "horizontal",
     "max",
     "grid",
+    NULL,
 };
 
 static void fetch_frame_colors() {
@@ -64,7 +66,7 @@ static void fetch_frame_colors() {
     g_swap_monitors_to_get_tag = &(settings_find("swap_monitors_to_get_tag")->value.i);
     g_direction_external_only = &(settings_find("default_direction_external_only")->value.i);
     g_gapless_grid = &(settings_find("gapless_grid")->value.i);
-    *g_default_frame_layout = CLAMP(*g_default_frame_layout, 0, LAYOUT_COUNT);
+    *g_default_frame_layout = CLAMP(*g_default_frame_layout, 0, LAYOUT_COUNT - 1);
     char* str = settings_find("frame_border_normal_color")->value.s;
     g_frame_border_normal_color = getcolor(str);
     str = settings_find("frame_border_active_color")->value.s;
@@ -81,7 +83,7 @@ static void fetch_frame_colors() {
         g_warning("too few characters in setting tree_style\n");
         // ensure that it is long enough
         char* argv[] = { "set", "tree_style", "01234567" };
-        settings_set(LENGTH(argv), argv);
+        settings_set_command(LENGTH(argv), argv);
     }
 }
 
@@ -148,8 +150,13 @@ void frame_insert_window(HSFrame* frame, Window window) {
         // append it to buf
         size_t count = frame->content.clients.count;
         count++;
+        // insert it after the selection
+        int index = frame->content.clients.selection + 1;
+        index = CLAMP(index, 0, count - 1);
         buf = g_renew(Window, buf, count);
-        buf[count-1] = window;
+        // shift other windows to the back to insert the new one at index
+        memmove(buf + index + 1, buf + index, sizeof(*buf) * (count - index - 1));
+        buf[index] = window;
         // write results back
         frame->content.clients.count = count;
         frame->content.clients.buf = buf;
@@ -210,7 +217,7 @@ bool frame_remove_window(HSFrame* frame, Window window) {
                 // if selection was before removed window
                 // then do nothing
                 // else shift it by 1
-                selection -= (selection <= i) ? 0 : 1;
+                selection -= (selection < i) ? 0 : 1;
                 // ensure, that it's a valid index
                 selection = count ? CLAMP(selection, 0, count-1) : 0;
                 frame->content.clients.selection = selection;
@@ -454,6 +461,7 @@ char* load_frame_tree(HSFrame* frame, char* description, GString** errormsg) {
             frame->content.clients.count = count;
 
             client->tag = tag;
+            ewmh_window_update_tag(client->window, client->tag);
 
             index++;
         }
@@ -476,6 +484,9 @@ char* load_frame_tree(HSFrame* frame, char* description, GString** errormsg) {
 
 int find_layout_by_name(char* name) {
     for (int i = 0; i < LENGTH(g_layout_names); i++) {
+        if (!g_layout_names[i]) {
+            break;
+        }
         if (!strcmp(name, g_layout_names[i])) {
             return i;
         }
@@ -627,6 +638,24 @@ int frame_current_cycle_client_layout(int argc, char** argv) {
     return 0;
 }
 
+int frame_current_set_client_layout(int argc, char** argv) {
+    int layout = 0;
+    if (argc <= 1) {
+        fprintf(stderr, "set_layout: not enough arguments\n");
+        return HERBST_INVALID_ARGUMENT;
+    }
+    layout = find_layout_by_name(argv[1]);
+    if (layout < 0) {
+        HSDebug("set_layout: invalid layout name \"%s\"\n", argv[1]);
+        return HERBST_INVALID_ARGUMENT;
+    }
+    if (g_cur_frame && g_cur_frame->type == TYPE_CLIENTS) {
+        g_cur_frame->content.clients.layout = layout;
+        monitor_apply_layout(get_current_monitor());
+    }
+    return 0;
+}
+
 void frame_apply_client_layout_linear(HSFrame* frame, XRectangle rect, bool vertical) {
     Window* buf = frame->content.clients.buf;
     size_t count = frame->content.clients.count;
@@ -842,13 +871,22 @@ HSMonitor* add_monitor(XRectangle rect, HSTag* tag) {
 
 int add_monitor_command(int argc, char** argv) {
     // usage: add_monitor RECTANGLE TAG [PADUP [PADRIGHT [PADDOWN [PADLEFT]]]]
-    if (argc < 3) {
+    if (argc < 2) {
         return HERBST_INVALID_ARGUMENT;
     }
     XRectangle rect = parse_rectangle(argv[1]);
-    HSTag* tag = find_tag(argv[2]);
-    if (!tag) {
-        return HERBST_INVALID_ARGUMENT;
+    HSTag* tag = NULL;
+    if (argc == 2 || !strcmp("", argv[2])) {
+        tag = find_unused_tag();
+        if (!tag) {
+            return HERBST_TAG_IN_USE;
+        }
+    }
+    else {
+        tag = find_tag(argv[2]);
+        if (!tag) {
+            return HERBST_INVALID_ARGUMENT;
+        }
     }
     if (find_monitor_with_tag(tag)) {
         return HERBST_TAG_IN_USE;
@@ -1008,6 +1046,44 @@ HSTag* find_tag(char* name) {
     return NULL;
 }
 
+int tag_index_of(HSTag* tag) {
+    for (int i = 0; i < g_tags->len; i++) {
+        if (g_array_index(g_tags, HSTag*, i) == tag) {
+            return i;
+        }
+    }
+    return -1;
+}
+
+HSTag* get_tag_by_index(char* index_str) {
+    int index = atoi(index_str);
+    // index must be treated relative, if it's first char is + or -
+    bool is_relative = array_find("+-", 2, sizeof(char), &index_str[0]) >= 0;
+    HSMonitor* monitor = get_current_monitor();
+    if (is_relative) {
+        int current = tag_index_of(monitor->tag);
+        index += current;
+        // ensure index is valid
+        index = MOD(index, g_tags->len);
+    } else {
+        // if it is absolute, then check index
+        if (index < 0 || index >= g_tags->len) {
+            HSDebug("invalid tag index %d\n", index);
+            return NULL;
+        }
+    }
+    return g_array_index(g_tags, HSTag*, index);
+}
+
+HSTag* find_unused_tag() {
+    for (int i = 0; i < g_tags->len; i++) {
+        if (!find_monitor_with_tag(g_array_index(g_tags, HSTag*, i))) {
+            return g_array_index(g_tags, HSTag*, i);
+        }
+    }
+    return NULL;
+}
+
 HSTag* add_tag(char* name) {
     HSTag* find_result = find_tag(name);
     if (find_result) {
@@ -1019,6 +1095,8 @@ HSTag* add_tag(char* name) {
     tag->name = g_string_new(name);
     tag->floating = false;
     g_array_append_val(g_tags, tag);
+    ewmh_update_desktops();
+    ewmh_update_desktop_names();
     tag_set_flags_dirty();
     return tag;
 }
@@ -1027,6 +1105,10 @@ int tag_add_command(int argc, char** argv) {
     if (argc < 2) {
         return HERBST_INVALID_ARGUMENT;
     }
+    if (!strcmp("", argv[1])) {
+        HSDebug("A empty tag name is not permitted\n");
+        return HERBST_INVALID_ARGUMENT;
+    }
     HSTag* tag = add_tag(argv[1]);
     hook_emit_list("tag_added", tag->name->str, NULL);
     return 0;
@@ -1044,6 +1126,7 @@ int tag_rename_command(int argc, char** argv) {
         return HERBST_TAG_IN_USE;
     }
     tag->name = g_string_assign(tag->name, argv[2]);
+    ewmh_update_desktop_names();
     hook_emit_list("tag_renamed", tag->name->str, NULL);
     return 0;
 }
@@ -1073,6 +1156,7 @@ int tag_remove_command(int argc, char** argv) {
     for (i = 0; i < count; i++) {
         HSClient* client = get_client_from_window(buf[i]);
         client->tag = target;
+        ewmh_window_update_tag(client->window, client->tag);
         frame_insert_window(target->frame, buf[i]);
     }
     if (monitor_target) {
@@ -1092,6 +1176,9 @@ int tag_remove_command(int argc, char** argv) {
             break;
         }
     }
+    ewmh_update_current_desktop();
+    ewmh_update_desktops();
+    ewmh_update_desktop_names();
     tag_set_flags_dirty();
     hook_emit_list("tag_removed", oldname, target->name->str, NULL);
     g_free(oldname);
@@ -1218,10 +1305,13 @@ int frame_current_cycle_selection(int argc, char** argv) {
         return 0;
     }
     int index = frame->content.clients.selection;
+    // use an integer variable to avoid strange happenings when computing
+    //       (-1) % (size_t)6
+    int count = (int) frame->content.clients.count;
     index += delta;
-    index %= frame->content.clients.count;
-    index += frame->content.clients.count;
-    index %= frame->content.clients.count;
+    index %= count;
+    index += count;
+    index %= count;
     frame->content.clients.selection = index;
     Window window = frame->content.clients.buf[index];
     window_focus(window);
@@ -2006,6 +2096,7 @@ void monitor_set_tag(HSMonitor* monitor, HSTag* tag) {
             frame_focus_recursive(tag->frame);
             monitor_apply_layout(other);
             monitor_apply_layout(monitor);
+            ewmh_update_current_desktop();
             emit_tag_changed(other->tag, other - (HSMonitor*)g_monitors->data);
             emit_tag_changed(tag, g_cur_monitor);
         }
@@ -2024,6 +2115,7 @@ void monitor_set_tag(HSMonitor* monitor, HSTag* tag) {
     // focus window just has been shown
     // focus again to give input focus
     frame_focus_recursive(tag->frame);
+    ewmh_update_current_desktop();
     emit_tag_changed(tag, g_cur_monitor);
 }
 
@@ -2039,28 +2131,57 @@ int monitor_set_tag_command(int argc, char** argv) {
     return 0;
 }
 
-int tag_move_window_command(int argc, char** argv) {
+int monitor_set_tag_by_index_command(int argc, char** argv) {
     if (argc < 2) {
         return HERBST_INVALID_ARGUMENT;
     }
-    HSFrame*  frame = g_cur_frame;
-    if (!g_cur_frame) {
-        // nothing to do
-        return 0;
+    HSTag* tag = get_tag_by_index(argv[1]);
+    if (!tag) {
+        return HERBST_INVALID_ARGUMENT;
     }
-    Window window = frame_focused_window(frame);
-    if (window == 0) {
-        // nothing to do
-        return 0;
+    monitor_set_tag(get_current_monitor(), tag);
+    return 0;
+}
+
+int tag_move_window_command(int argc, char** argv) {
+    if (argc < 2) {
+        return HERBST_INVALID_ARGUMENT;
     }
     HSTag* target = find_tag(argv[1]);
     if (!target) {
         return HERBST_INVALID_ARGUMENT;
     }
+    tag_move_window(target);
+    return 0;
+}
+
+int tag_move_window_by_index_command(int argc, char** argv) {
+    if (argc < 2) {
+        return HERBST_INVALID_ARGUMENT;
+    }
+    HSTag* tag = get_tag_by_index(argv[1]);
+    if (!tag) {
+        return HERBST_INVALID_ARGUMENT;
+    }
+    tag_move_window(tag);
+    return 0;
+}
+
+void tag_move_window(HSTag* target) {
+    HSFrame*  frame = g_cur_frame;
+    if (!g_cur_frame) {
+        // nothing to do
+        return;
+    }
+    Window window = frame_focused_window(frame);
+    if (window == 0) {
+        // nothing to do
+        return;
+    }
     HSMonitor* monitor = get_current_monitor();
     if (monitor->tag == target) {
         // nothing to do
-        return 0;
+        return;
     }
     HSMonitor* monitor_target = find_monitor_with_tag(target);
     frame_remove_window(frame, window);
@@ -2069,6 +2190,8 @@ int tag_move_window_command(int argc, char** argv) {
     HSClient* client = get_client_from_window(window);
     assert(client != NULL);
     client->tag = target;
+    ewmh_window_update_tag(client->window, client->tag);
+
     // refresh things
     if (monitor && !monitor_target) {
         // window is moved to unvisible tag
@@ -2081,7 +2204,6 @@ int tag_move_window_command(int argc, char** argv) {
         monitor_apply_layout(monitor_target);
     }
     tag_set_flags_dirty();
-    return 0;
 }
 
 int monitor_focus_command(int argc, char** argv) {
@@ -2156,6 +2278,7 @@ void monitor_focus_by_index(int new_selection) {
         XWarpPointer(g_display, None, g_root, 0, 0, 0, 0, new_x, new_y);
     }
     // emit hooks
+    ewmh_update_current_desktop();
     emit_tag_changed(monitor->tag, new_selection);
 }
 
