@@ -1,4 +1,4 @@
-/** Copyright 2011 Thorsten Wißmann. All rights reserved.
+/** Copyright 2011-2012 Thorsten Wißmann. All rights reserved.
  *
  * This software is licensed under the "Simplified BSD License".
  * See LICENSE for details */
@@ -16,6 +16,7 @@
 #include "hook.h"
 #include "mouse.h"
 #include "rules.h"
+#include "ewmh.h"
 // standard
 #include <string.h>
 #include <stdio.h>
@@ -60,7 +61,6 @@ int custom_hook_emit(int argc, char** argv);
 // handler for X-Events
 void buttonpress(XEvent* event);
 void buttonrelease(XEvent* event);
-void clientmessage(XEvent* event);
 void createnotify(XEvent* event);
 void configurerequest(XEvent* event);
 void configurenotify(XEvent* event);
@@ -82,6 +82,7 @@ CommandBinding g_commands[] = {
     CMD_BIND(             "version",        version),
     CMD_BIND(             "list_commands",  list_commands),
     CMD_BIND(             "list_monitors",  list_monitors),
+    CMD_BIND(             "list_keybinds",  key_list_binds),
     CMD_BIND_NO_OUTPUT(   "keybind",        keybind),
     CMD_BIND_NO_OUTPUT(   "keyunbind",      keyunbind),
     CMD_BIND_NO_OUTPUT(   "mousebind",      mouse_bind_command),
@@ -98,13 +99,15 @@ CommandBinding g_commands[] = {
     CMD_BIND_NO_OUTPUT(   "focus",          frame_focus_command),
     CMD_BIND_NO_OUTPUT(   "shift",          frame_move_window_command),
     CMD_BIND_NO_OUTPUT(   "remove",         frame_remove_command),
-    CMD_BIND_NO_OUTPUT(   "set",            settings_set),
+    CMD_BIND_NO_OUTPUT(   "set",            settings_set_command),
     CMD_BIND_NO_OUTPUT(   "toggle",         settings_toggle),
+    CMD_BIND_NO_OUTPUT(   "cycle_value",    settings_cycle_value),
     CMD_BIND_NO_OUTPUT(   "cycle_monitor",  monitor_cycle_command),
     CMD_BIND_NO_OUTPUT(   "focus_monitor",  monitor_focus_command),
     CMD_BIND(             "get",            settings_get),
     CMD_BIND_NO_OUTPUT(   "add",            tag_add_command),
     CMD_BIND_NO_OUTPUT(   "use",            monitor_set_tag_command),
+    CMD_BIND_NO_OUTPUT(   "use_index",      monitor_set_tag_by_index_command),
     CMD_BIND(             "floating",       tag_set_floating_command),
     CMD_BIND_NO_OUTPUT(   "fullscreen",     client_set_property_command),
     CMD_BIND_NO_OUTPUT(   "pseudotile",     client_set_property_command),
@@ -112,6 +115,7 @@ CommandBinding g_commands[] = {
     CMD_BIND_NO_OUTPUT(   "merge_tag",      tag_remove_command),
     CMD_BIND_NO_OUTPUT(   "rename",         tag_rename_command),
     CMD_BIND_NO_OUTPUT(   "move",           tag_move_window_command),
+    CMD_BIND_NO_OUTPUT(   "move_index",     tag_move_window_by_index_command),
     CMD_BIND_NO_OUTPUT(   "add_monitor",    add_monitor_command),
     CMD_BIND_NO_OUTPUT(   "remove_monitor", remove_monitor_command),
     CMD_BIND_NO_OUTPUT(   "move_monitor",   move_monitor_command),
@@ -417,8 +421,7 @@ void scan(void) {
             // only manage mapped windows.. no strange wins like:
             //      luakit/dbus/(ncurses-)vim
             // TODO: what would dwm do?
-            if (!is_window_ignored(wins[i]) &&
-                is_window_mapped(g_display, wins[i])) {
+            if (is_window_mapped(g_display, wins[i])) {
                 manage_client(wins[i]);
             }
         }
@@ -476,7 +479,7 @@ static void parse_arguments(int argc, char** argv) {
                 break;
             case 'v':
                 printf("%s %s\n", argv[0], HERBSTLUFT_VERSION);
-                printf("Copyright (c) 2011 Thorsten Wißmann\n");
+                printf("Copyright (c) 2011-2012 Thorsten Wißmann\n");
                 printf("Released under the Simplified BSD License\n");
                 exit(0);
                 break;
@@ -510,23 +513,23 @@ static void fetch_settings() {
 }
 
 static HandlerTable g_default_handler = {
-    [ButtonPress] = buttonpress,
-    [ButtonRelease] = buttonrelease,
-    [ClientMessage] = clientmessage,
-    [CreateNotify] = createnotify,
-    [ConfigureRequest] = configurerequest,
-    [ConfigureNotify] = configurenotify,
-    [DestroyNotify] = destroynotify,
-    [EnterNotify] = enternotify,
-    [Expose] = expose,
-    [FocusIn] = focusin,
-    [KeyPress] = keypress,
-    [MappingNotify] = mappingnotify,
-    [MotionNotify] = motionnotify,
-    [MapNotify] = mapnotify,
-    [MapRequest] = maprequest,
-    [PropertyNotify] = propertynotify,
-    [UnmapNotify] = unmapnotify,
+    [ ButtonPress       ] = buttonpress,
+    [ ButtonRelease     ] = buttonrelease,
+    [ ClientMessage     ] = ewmh_handle_client_message,
+    [ CreateNotify      ] = createnotify,
+    [ ConfigureRequest  ] = configurerequest,
+    [ ConfigureNotify   ] = configurenotify,
+    [ DestroyNotify     ] = destroynotify,
+    [ EnterNotify       ] = enternotify,
+    [ Expose            ] = expose,
+    [ FocusIn           ] = focusin,
+    [ KeyPress          ] = keypress,
+    [ MappingNotify     ] = mappingnotify,
+    [ MotionNotify      ] = motionnotify,
+    [ MapNotify         ] = mapnotify,
+    [ MapRequest        ] = maprequest,
+    [ PropertyNotify    ] = propertynotify,
+    [ UnmapNotify       ] = unmapnotify,
 };
 
 static struct {
@@ -538,6 +541,7 @@ static struct {
     { settings_init,    settings_destroy    },
     { clientlist_init,  clientlist_destroy  },
     { layout_init,      layout_destroy      },
+    { ewmh_init,        ewmh_destroy        },
     { mouse_init,       mouse_destroy       },
     { hook_init,        hook_destroy        },
     { rules_init,       rules_destroy       },
@@ -573,9 +577,7 @@ void buttonrelease(XEvent* event) {
     HSDebug("name is: ButtonRelease\n");
     mouse_stop_drag();
 }
-void clientmessage(XEvent* event) {
-    HSDebug("name is: ClientMessage\n");
-}
+
 void createnotify(XEvent* event) {
     // printf("name is: CreateNotify\n");
     if (is_ipc_connectable(event->xcreatewindow.window)) {
@@ -626,17 +628,19 @@ void motionnotify(XEvent* event) {
 }
 void mapnotify(XEvent* event) {
     HSDebug("name is: MapNotify\n");
-    if (get_client_from_window(event->xmap.window)) {
+    HSClient* c;
+    if ((c = get_client_from_window(event->xmap.window))) {
         // reset focus. so a new window gets the focus if it shall have the
         // input focus
         frame_focus_recursive(g_cur_frame);
+        // also update the window title - just to be sure
+        client_update_title(c);
     }
 }
 void maprequest(XEvent* event) {
     HSDebug("name is: MapRequest\n");
     XMapRequestEvent* mapreq = &event->xmaprequest;
-    if (is_window_ignored(mapreq->window)
-        || is_herbstluft_window(g_display, mapreq->window)) {
+    if (is_herbstluft_window(g_display, mapreq->window)) {
         // just map the window if it wants that
         XWindowAttributes wa;
         if (!XGetWindowAttributes(g_display, mapreq->window, &wa)) {
@@ -667,6 +671,9 @@ void propertynotify(XEvent* event) {
                 case XA_WM_HINTS:
                     client_update_wm_hints(client);
                     break;
+                case XA_WM_NAME:
+                    client_update_title(client);
+                    break;
                 default:
                     break;
             }
@@ -708,6 +715,7 @@ int main(int argc, char* argv[]) {
     scan();
     tag_force_update_flags();
     all_monitors_apply_layout();
+    ewmh_update_all();
     execute_autostart_file();
 
     // main loop
