@@ -1,4 +1,4 @@
-/** Copyright 2011 Thorsten Wißmann. All rights reserved.
+/** Copyright 2011-2012 Thorsten Wißmann. All rights reserved.
  *
  * This software is licensed under the "Simplified BSD License".
  * See LICENSE for details */
@@ -19,6 +19,7 @@ typedef struct {
         int     i;
         char*   s;
     }   value;
+    int old_value_i;
     int type;
     void (*on_change)(); // what to call on change
 } SettingsPair;
@@ -30,8 +31,10 @@ void settings_destroy();
 
 SettingsPair* settings_find(char* name);
 
-int settings_set(int argc, char** argv);
+int settings_set(SettingsPair* pair, char* value);
+int settings_set_command(int argc, char** argv);
 int settings_toggle(int argc, char** argv);
+int settings_cycle_value(int argc, char** argv);
 int settings_count();
 int settings_get(int argc, char** argv, GString** output);
 
